import { z } from 'zod'
import { hasNonEmptyIntersection } from '@/utils'

export const EmptyOrFilled = z.string().transform((val) => !!val.trim())

export const UsageArea = z.enum(['Lab Bench', 'Automation'])
export type UsageArea = z.infer<typeof UsageArea>
export const ALL_USAGE_AREAS = UsageArea._def.values

export const Labware = z.enum([
  'TubesVialsLarge',
  'TubesVialsSmall',
  'Microplates',
  'MicroplatesHighDensity',
  'MicroplatesMediumDensity',
  'MicroplatesLowDensity',
  'DeepWellPlates',
])
export type Labware = z.infer<typeof Labware>
export const ALL_LABWARES = Labware._def.values

export const Method = z.enum(['Shake', 'Heat', 'Cool', 'Tilt', 'Turn'])
export type Method = z.infer<typeof Method>
export const ALL_METHODS = Method._def.values

export const METHOD_COLOR_MAP: Record<Method, string> = {
  Shake: '#000000',
  Heat: '#EC6602',
  Cool: '#3A5799',
  Tilt: '#757776',
  Turn: '#AAAAAA',
}
export const getMethodColor = (method: Method): string => METHOD_COLOR_MAP[method]

export const TechSpec = z.string().brand('TechSpech')
export type TechSpec = z.infer<typeof TechSpec>

export const Product = z.object({
  name: z.string(),
  partNo: z.string(),
  areas: z.array(UsageArea),
  labwares: z.array(Labware),
  methods: z.array(Method),
  techSpecs: z.array(TechSpec),
  adapterRequired: z.boolean().optional(),
  magneticBeadsRequired: z.boolean().optional(),
  fgColor: z.string(),
  bgColor: z.string(),
  desc: z.string().nullish(),
  imageUrl: z.string().nullish(),
})
export type Product = z.infer<typeof Product>

export interface ProductFilters {
  areas: UsageArea[]
  labwares: Labware[]
  methods: Method[]
  techSpecs: TechSpec[]
}

export const isProductSelected = (product: Product, filters: ProductFilters): boolean =>
  hasNonEmptyIntersection(product.areas, filters.areas) &&
  hasNonEmptyIntersection(product.labwares, filters.labwares) &&
  (filters.methods.length === ALL_METHODS.length ||
    filters.methods.every((method) => product.methods.includes(method))) &&
  (filters.techSpecs.length === 0 ||
    filters.techSpecs.every((techSpec) => product.techSpecs.includes(techSpec)))
